package bccpbmod;

# bccpbmod.pm - Common module defining global variables & shared subroutines
#
# Authors:  Brandt Onorato, Steve Seifert, Tony Lee, Rich Ralston, Ray Green
# Last updated:  Steve Seifert, 9/14/06
#		     Jane Huang, 7/15/07  
#
# (c) Copyright IBM Corp. 2008
#
# This is the common routines used in all the Build scripts.
#
# You must modify this script before attempting to use parameterized build.
# Sections that require modification and the instructions for modifying them
# are flagged with the following comment lines:
#
# ******************************************************************************
# * Customization instructions:                                                *
# * [instructions here]                                                        *
# ****************************************************************************** 



# Share global variables with other modules
use vars qw($debug);
use vars qw($PROD_stream, $QA_stream, $DEV_stream);
use vars qw($PROD_env,    $QA_env,    $DEV_env);
use vars qw($rccbuild, $sysout);

# Define local variables
$rccbuild = 'rccbuild.exe';
$sysout   = 'SYSOUT=*';

$debug = 1;

$PROD_env   = "PROD";
$QA_env     = "QA";
$DEV_env    = "DEV";

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<  User ID Table  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# getUserID uses a lookup table to translate the users' PC username to their
# mainframe ID.  
#
# This routine could be rewritten query a database for the mainframe ID.

sub getUserID {
	my ($pcid) = @_;

	$pcid = lc $pcid;
      printf("*** bccpbmod::getUserID ***\n") if $debug;
	printf( "PCID from getUserID: (%s)\n", $pcid ) if $debug;


# *******************************************************************************
# * Required customization:                                                     *
# * The user Id table 'userid' is used in this example. It must be modified to  *
# * contain all authorized ids for performing the parameterized build.  Review  *
# * the document for details of setting the user id table.                      *
# * The location of the user Id table can be set with a type attribute SCRIPTDIR*
# * in PVOB or hardcoded here.                                                  *
# ******************************************************************************* 

my $hostid= "";       # User ID Table 

# Get the build script directory from type attribute SCRIPTDIR first  

my $project = `cleartool lsstream -fmt \"%[project]Xp\"`;      
my $scriptfile = `cleartool desc -fmt \"%[SCRIPTDIR]SNa\" $project`;

  
    if ($scriptfile) {
   	$scriptfile =~ s/^\"//;  # remove quotes around attr value
   	$scriptfile =~ s/\"$//;  # returned from cleartool cmd
      $hostid = $scriptfile ."\\userid";              
    } else {
      $hostid= "\\\\PFHUANG\\ccstg_d\\Triggers\\userid";
    }
  
  
my $pcname ='';
my $mvsname ='';
my $line =''; 
my $tab = "\t";   # tab by default
my $mfid = '';

open(USERIDS, $hostid) or die "Can't open userid: $!\n";
 while ($line = <USERIDS>) {
     chomp $line;
     ($pcname, $mvsname) = split(/$tab/, $line);
  
    if ($pcname eq $pcid) {
          $mfid = $mvsname;
     print "Get mfid from userid table : $mfid \n";
  } 
 }
 # print "Get mfid from userid table: $mfid \n";  
	#my %username = (
	#	'brandt.onorato' => 'PSJ5224',
	#	'mwaldner'	=> 'PSB2248',
	#	'pstandley' => 'PSJ5122',
 	#     'jphuang'  => 'JHUANG',
	#);

	#return $username{$pcid};
     return $mfid;  
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< setEnvironmentVar >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# setEnvironmentVar returns the attribute values attached to a CC UCM Project.  The
# Project is determined by the input parameter "$stream" or if the parameter is 
# not provided, by the stream used in the context of the current operation.

sub setEnvironmentVar {
    my ($stream) = @_;
   
    printf("*** bccpbmod::setEnvironmentVar ***\n") if $debug;
    # printf( "STREAM:(%s)\n", $stream ) if $debug;
    # printf( "  PROD ENV   :(%s)\n", $PROD_env ) if $debug;
    # printf( "  QA   ENV   :(%s)\n", $QA_env ) if $debug;
    # printf( "  DEV  ENV   :(%s)\n", $DEV_env ) if $debug;

    my $streamType = "";
       
    # printf( "  SVL HOST    :(%s)\n", $os390_hostname ) if $debug;

    my @env;    # $env, $db2sys, $menv, $os390_hostname, $os390_port, $stage
                # $copylib1, $copylib2, $copylib3, $copylib4;
                # $syslib1, $syslib2, $syslib3;
                # $syslib4, $steplib1, $steplib2;		      # Optional
                # $hostpds_prefix, $hostpds_affix, $cbllib;	      # Optional 
                # $loadlib, $c390llib, $dbrmlib, $tgtlib, $db2_owner

           
# Identify the CC View context of the operation
    my $my_view = $ENV{"CLEARCASE_VIEW_TAG"};  #  try trigger env variable
#   printf( "  MY VIEW from CC ENV:(%s)\n", $my_view ) if $debug; 
   
    if ($my_view eq "") {
    	$my_view = `cleartool lsview -cview -short`;  # if not trigger try cmd line/GUI
    		if ($my_view eq "") {
		    	print "Can't get view.\n";
       			 exit -1;
		}
    }
    chomp $my_view;
    printf( "VIEW:(%s)\n", $my_view ) if $debug;

# Identify the CC Stream context of the operation or used the input parameter
    if ($stream eq "") {

    	$stream = `cleartool lsstream -s -view $my_view`;

    	if ($stream eq "") {
    		print "Can't get stream.\n";
        	exit -1;
    	}
    	chomp $stream;
    	print "Current Stream: \"$stream\"\n" if $debug;

    	my $intstream = `cleartool lsproject -fmt "%[istream]p" -view $my_view`;
    	if ($intstream eq "") {
    		print "Can't get integration stream.\n";
        	exit -1;
    	}
    	chomp $intstream;
    	print "Integration Stream: \"$intstream\"\n" if $debug;
     
    	if ($stream eq $intstream) {
    		$streamType = "INT";
    	} else {
		$streamType = "DEV";
    	}

 #    $stream .= "@\\pvob_parmbld";    
      } else {
    	print "Input Stream: \"$stream\"\n" if $debug;
    }


# Identify the CC Project 
# my $project = `cleartool desc -fmt \"%[project]p\" stream:$stream`;  
  my $project = `cleartool lsstream -fmt \"%[project]Xp\"`;      

  printf( "Project :(%s)\n", $project ) if $debug;    

# Retrieve the Project attribute values or exit if the Project is not Z-Enabled
# my $env_attr = `cleartool desc -fmt \"%[ZCCENV]SNa\" project:$project@\\pvob_parmbld`;
  my $env_attr = `cleartool desc -fmt \"%[ZCCENV]SNa\" $project`;

   # printf( "env_attr:(%s)\n", $env_attr ) if $debug;    

    my $spt = ",";
    if ($env_attr) {
   	$env_attr =~ s/^\"//;  # remove quotes around attr value
   	$env_attr =~ s/\"$//;  # returned from cleartool cmd
    	@env = split(/,/,$env_attr);  # comment out test only
 
    
    	if ($streamType eq "DEV") {  # allows delivery from dev stream to in stream
		$env[2] = $env[0];   # in this case the to env is still DEV
	}
    } else {
    	bccpbmod::my_die("Project is not Z enabled\n");
    }

# Return the array of values.
    printf( "Environment name=(%s)\n", $env[0] ) if $debug;
    return @env;
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< writeFile >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# writeFile formats and writes a JCL deck to a file

sub writeFile {

    my ( $outputfilename, $compile_1, $text ) = @_;

    print "OUTPUTFILE: $outputfilename \n" if $debug;
    open( FOUT, ">> $outputfilename" ) or die 'Unable to open output file';

    my $n = length($text);

    my $first = substr( $text, 0, 1 );
    my $last;
    if ( $n > 2 ) {
        $last = substr( $text, $n - 2, 2 );
        if ( $last eq "\n\n" ) {
            chop($text);
        }
    }
    if ( $first eq "\n" ) {
        $text = substr( $text, 1 );
    }

    if ($compile_1) {
        print STDERR $text;
    }
    print FOUT $text;

    close(FOUT);

}


# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< devPrint >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
# 
# devPrint builds an identification string from the CC version information
# that is embedded into the file.  This information can be used to trace
# executables back to source code and deployment.

sub devPrint {

    my ($env,$mem,$mfid,$filename) = @_;    # get the filename which is passed as argument

    my $version = `cleartool desc -fmt \"%Ln\" $filename`;
    print "Leaf number: $version\n" if $debug;

    $id = "$env.$version.$mem.$mfid";
    print "ID: $id\n" if $debug;

    return ($id);
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< getCCattrs >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# getCCattrs retrieves the attributes from the element in name/value pairs and returns
# an array where the array element key is the attribute name which contains the value.

sub getCCattrs {

    my ($filename) = @_;    # get the filename which is passed as argument
    print("*** bccpbmod::getCCattrs ***\n") if $debug;

# get the list of CC attributes attached to the file element -  not version, i.e. (a='N', b='Y')
    my $ret = `cleartool desc -fmt \"%a\" $filename\@\@`;
        
    chomp $ret;    # remove the return char
    chop $ret;     # remove the closing parenthesis
    $ret = substr( $ret, 1 );    # remove the opening parenthesis
    my @attrs = split /, /, $ret;    # split the line into multiple lines of a='N'
    my ( $pair, $name, $value );
    my %CCattr;    # associative array for attribute name-value pairs

    # foreach line, a='N', split out the name and value
    foreach $pair (@attrs) {
        ( $name, $value ) = split /=/, $pair;
        $value =~ s/\"//g;         # remove double quotes
        $CCattr{$name} = $value;   # assign name-value pair to attributes array
    }

    # fix any old code missing the SP attribute
#    if (   ( $CCattr{'DB2'} eq 'Y' )
#        || ( substr( $mem, 5, 1 ) eq 'T' )
#        || ( $mem =~ /^CSXXS0((07)|(08)|(11))$/ ) )
#    {
#        $CCattr{'SP'} = 'Y';
#    }

    # return the associative array back to the callee code
    return %CCattr;
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< testServer >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# testServer performs a specific rccbuild command that verifies communication
# with the mainframe.

sub testServer {
    my ($os390_hostname, $os390_port) = @_;
    my $rccbuild = 'rccbuild.exe';

    printf("Testing if the server is up...");

    my $cmd = "$rccbuild -h $os390_hostname\@$os390_port -testServer";

    printf("Executing ($cmd)\n") if $debug;

    my $status = 0;

#   $status = cofkymodrcc_system($compile_1, $cmd);
    $status = rcc_system($compile_1, $cmd);

    printf( "Status was(%d)\n", $status ) if $debug;

    if ( $status == 256 ) {
        die
"\n\nUnable to connect to server $os390_hostname\@$os390_port. Aborting compile. \n";
    }
}


# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< cleanup >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# cleanup removes JCL files from a previous execution

sub cleanup {
    my ($logdir) = @_;

    if (-d $logdir) {
        unlink("rccbuild.log");
        unlink("$logdir/devprint.jcl");
        unlink("$logdir/s1delete.jcl");
        unlink("$logdir/s1alloc.jcl");
        unlink("$logdir/ctranslt.jcl");
        unlink("$logdir/predb2.jcl");
        unlink("$logdir/resolvecb.jcl");
        unlink("$logdir/compile.jcl");
        unlink("$logdir/sidedeck.jcl");
        unlink("$logdir/prelink.jcl");
        unlink("$logdir/linksp.jcl");
        unlink("$logdir/copysp.jcl");
        unlink("$logdir/lnkbatch.jcl");
        unlink("$logdir/copybt.jcl");
        unlink("$logdir/copydo.jcl");
        unlink("$logdir/amblist.jcl");
        unlink("$logdir/movdbrm.jcl");
        unlink("$logdir/db2bind.jcl");
        unlink("$logdir/lnkcics.jcl");
        unlink("$logdir/copycs.jcl");
        unlink("$logdir/genbnd.jcl");
        unlink("$logdir/perfbnd.jcl");
        unlink("$logdir/generateSPCopy.jcl");
        unlink("$logdir/buildSPCopy.jcl");
        unlink("$logdir/executeSPCopy.jcl");
        unlink("$logdir/cicsNewcopy.jcl");
    } else {
        mkdir $logdir;
    }
}
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< cleanup >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#  Cleanup parts for parmbld_main
# cleanup_parts removes part of JCL files from a previous execution

sub cleanup_parts {
    my ($logdir) = @_;

    if (-d $logdir) {
        unlink("$logdir/lnkbatch.jcl");
    } else {
        mkdir $logdir;
    }
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< rcc_system >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# rcc_system executes an rcc command on the mainframe and returns the status

sub rcc_system {
	my ($compile_1, $cmd) = @_;

	my $rcc_out = "rcc.out";
	my $status = 0;
	$cmd .= " > $rcc_out";

	$status = system($cmd);

	open(RCCOUT, $rcc_out) || die "Can't open $rcc_out\n";
	while (<RCCOUT>) {
		print;
		if ($compile_1) {		# SDS 0801
			print STDERR;
		} else {
			print STDERR if /^The MVS step.*return code is/;
		}
	}
	close RCCOUT;
	unlink($rcc_out);

	return $status;
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< pause >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# pause waits for the user to press any key

sub pause {

	print "Press enter to exit.\n";
	my $ignore = <STDIN>;
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< printArray >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# printArray the values of an array

sub printArray {

	foreach $name (@_) {
		print "\t$name\n";
	}
	return 0;
}

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< my_die >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# my_die exits a script printing the error message that caused the error to stdout
#
sub my_die {
	die "\nFATAL ERROR: @_ \n";
 exit; 
}


